@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998  Dimitrios P. Bouras
+  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,13 +26,14 @@
 #include <net/if.h>
 #include <arpa/inet.h>
 #include <string.h>
+#include <sys/param.h>
 #include <unistd.h>
 
 #ifdef SUNOS41x
  extern int socket(), ioctl(), sscanf(), printf(), pclose();
 #elif defined(SUNOS5x)
  #include <sys/sockio.h>
-#else
+#elif defined(linux)
 
 /* Returns kernel revision number (10*major+minor). Only used on Linux to
    figure out what kind of parsing format to use for /proc/net/dev output */
@@ -88,10 +89,14 @@ char *IFAddr(char *IFName, int dest)
  #define AddrOpen   popen("/usr/ucb/netstat -nr", "r")
  #define AddrScanf  sscanf(routeLine, "%*s %*s %*s %*s %*s %s", IF)
  #define AddrClose  while (fgets(routeLine,128,infofp) != NULL); pclose(infofp)
-#elif defined(SUNOS5x)
+#elif defined(SUNOS5x) || defined(__FreeBSD__)
  #define AddrOpen   popen("/usr/bin/netstat -nr", "r")
  #define AddrScanf  sscanf(routeLine, "%*s %*s %*s %*s %*s %s", IF)
  #define AddrClose  while (fgets(routeLine,128,infofp) != NULL); pclose(infofp)
+#elif (defined(BSD) && BSD >= 199306)
+ #define AddrOpen   popen("/usr/bin/netstat -nr", "r")
+ #define AddrScanf  sscanf(routeLine, "%*s %*s %*s %*s %*s %*s %s", IF)
+ #define AddrClose  while (fgets(routeLine,128,infofp) != NULL); pclose(infofp)
 #else
  #define AddrOpen   fopen("/proc/net/route", "r")
  #define AddrScanf  sscanf(routeLine,"%s", IF)
@@ -137,12 +142,18 @@ char *pppAddr(char *IFName)
  #define PktsScanf sscanf(line, "%s %*s %*s %*s %u %*s %u", \
 						  interface, &recv, &trans)
  #define PktsClose while (fgets(line,128,infofp) != NULL); pclose(infofp)
+#elif (defined(BSD) && BSD >= 199306)
+ #define PktsOpen  popen((sprintf(line,"/usr/bin/netstat -n -I %s",IFName)) ? \
+						 line:line, "r")
+ #define PktsScanf sscanf(line, "%s %*s %s %*s %u %*s %u", \
+						  interface, network, &recv, &trans)
+ #define PktsClose while (fgets(line,128,infofp) != NULL); pclose(infofp)
 #else
  #define PktsOpen  fopen("/proc/net/dev", "r")
  #define PktsScanf sscanf(line, ((kernelRevNo()>20) ? \
-						  "%s %*s %u %*s %*s %*s %*s %*s %u": \
-						  "%s %u %*s %*s %*s %*s %u"), \
-						  interface, &recv, &trans)
+				  " %[^ \t\r\n:] %*c %*s %u %*s %*s %*s %*s %*s %*s %*s %u" : \
+				  " %[^ \t\r\n:] %*c %u %*s %*s %*s %*s %u"), \
+				  interface, &recv, &trans)
  #define PktsClose fclose(infofp)
 #endif
 
@@ -151,16 +162,26 @@ unsigned pppPkts(char *IFName, unsigned long *pTX, unsigned long *pRX)
     int IFLen = strlen(IFName);
     FILE *infofp;
     char line[129], interface[16];
+#if (defined(BSD) && BSD >= 199306)
+    char network[16];
+#endif
     unsigned recv = 0, trans = 0;
 
 	infofp = PktsOpen;							/* open IF stats info stream */
     while (fgets(line, 128, infofp) != NULL) {
 		PktsScanf;								/* read line by line */
 		if (!strncmp(interface,IFName,IFLen)) {	/* scanning for interface */
+#if (defined(BSD) && BSD >= 199306)
+			/* NetBSD: discard non-conforming multicast line. */
+			if (strncmp(network,"<Link>", strlen(network))) {
+#endif
 			PktsClose;							/* found, close info stream */
 			if (pTX) *pTX = trans;				/* if return storage avail */
 			if (pRX) *pRX = recv;				/* save TXed/RXed separately */
 			return (recv + trans);				/* return total I/O packets */
+#if (defined(BSD) && BSD >= 199306)
+			}
+#endif
 		}
 	}
 	PktsClose;									/* close info stream */
