@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998  Dimitrios P. Bouras
+  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -155,13 +155,13 @@ static void doErr(char *msg)
 {
 	char emsg[MSGLEN_ERR+1];
 
-#ifndef SUNOS5x
+#ifdef HAVE_STRERROR
+	sprintf(emsg, "xISP: %s: %s\n", msg, strerror(errno));
+#else
 	if (errno < sys_nerr)
 		sprintf(emsg, "xISP: %s: %s\n", msg, sys_errlist[errno]);
 	else
 		sprintf(emsg, "xISP: %s: error #%d\n", msg, errno);
-#else
-	sprintf(emsg, "xISP: %s: %s\n", msg, strerror(errno));
 #endif
 	fputs(emsg, stderr);
 	exit(1);
@@ -198,7 +198,7 @@ static void outputAllXisprc(xisprc_t *p, FILE *rcfp, glob_t *global)
 	rcPrint(rcfp, "NUMPTTS: %d\n", global->numPTTs);
 	rcPrint(rcfp, "QTHPTT: %d\n", global->dfltPTT);
 	rcPrint(rcfp, "QTHZONE: %d\n", global->dfltZone);
-	rcPrint(rcfp, "LOGOPTS: %X\n", global->logOpts);
+	rcPrint(rcfp, "LOGOPTS: %hX\n", global->logOpts);
 	rcPrint(rcfp, "PPPDPATH: %s\n", global->pppdPath);
 	rcPrint(rcfp, "RUNPATH: %s\n", global->runPath);
 	rcPrint(rcfp, "CHATPATH: %s\n", global->chatPath);
@@ -233,12 +233,15 @@ static void outputAllXisprc(xisprc_t *p, FILE *rcfp, glob_t *global)
 		}
 		rcPrint(rcfp, "TERMW: %d\n", p->termW);
 		rcPrint(rcfp, "TERMH: %d\n", p->termH);
+		rcPrint(rcfp, "CBTERMW: %d\n", p->CBtermW);
+		rcPrint(rcfp, "CBTERMH: %d\n", p->CBtermH);
+		rcPrint(rcfp, "CBPHONE: %s\n", p->CBphone);
 		rcPrint(rcfp, "MODEMDEVICE: %s\n", p->modemDevice);
 		rcPrint(rcfp, "MODEMSPEED: %s\n", speed2str(p->modemSpeed));
 		rcPrint(rcfp, "MODEMRESET: %s\n", p->modemReset);
 		rcPrint(rcfp, "MODEMINIT: %s\n", p->modemInit);
 		rcPrint(rcfp, "MODEMCONNECT: %s\n", p->modemConnect);
-		rcPrint(rcfp, "OPEROPTS: %X\n", p->operOpts);
+		rcPrint(rcfp, "OPEROPTS: %lX\n", p->operOpts);
 		rcPrint(rcfp, "DIALEXTRA: %s\n", p->dialExtra);
 		rcPrint(rcfp, "COMPLEVEL: %d\n", p->compLevel);
 		rcPrint(rcfp, "ASYNCMAP: %s\n", p->asyncmap);
@@ -298,17 +301,18 @@ static void rcLineError(char *function, char *line)
 }
 
 /* Reads parameters from .xisprc file, doing error checking. NOTE: it
-   assumes a line less than 256 bytes and a parameter less than 128 */
+   assumes a line less than 512 bytes and a parameter less than 256 */
 
-#define MAXLEN_LINE 256
-#define MAXLEN_PARAM 128
+#define MAXLEN_LINE 512
+#define MAXLEN_PARAM 256
 
 static char *readParam(FILE *fp, char *name, char type,
 					   int llimit, int ulimit, void *data)
 {
 	static char line[MAXLEN_LINE];
 	char sparam[MAXLEN_PARAM] = {0}, *p, *endp;
-	int iparam, hex = 0, len;
+	long iparam;
+	int len, hex = 0;
 
 	if (fgets(line, MAXLEN_LINE, fp) == NULL) {
 		fprintf(stderr, "xISP: readParam: %s: premature EOF\n", rcfn);
@@ -333,15 +337,21 @@ static char *readParam(FILE *fp, char *name, char type,
 			break;
 
 		case 'X':
+		case 'L':
 			hex = 1;
 		case 'B':
 		case 'I':
 			iparam = strtol(sparam, &endp, (hex)? 16:10);
-			if ((!hex && (ulimit<iparam || iparam<llimit)) ||
-				 endp == sparam || *endp )
-				rcLineError("readParam", line);
-			if (type == 'B') *((unsigned char *)data) = iparam;
-			else *((unsigned int *)data) = iparam;
+			if (hex) {
+				if (type == 'X') *((unsigned short *)data) = iparam;
+				else *((unsigned long *)data) = iparam;
+			}
+			else {
+				if (ulimit<iparam || iparam<llimit || endp == sparam || *endp)
+					rcLineError("readParam", line);
+				if (type == 'B') *((unsigned char *)data) = iparam;
+				else *((unsigned int *)data) = iparam;
+			}
 			break;
 
 		default: break;
@@ -414,6 +424,9 @@ static void inputAllXisprc(xisprc_t *p, FILE *rcfp, glob_t *global)
 		}
 		readParam(rcfp, "TERMW", 'B', MINCHAR_TERMW, 255, &(p->termW));
 		readParam(rcfp, "TERMH", 'B', MINCHAR_TERMH, 255, &(p->termH));
+		readParam(rcfp, "CBTERMW", 'B', MINCHAR_TERMW, 255, &(p->CBtermW));
+		readParam(rcfp, "CBTERMH", 'B', MINCHAR_TERMH, 255, &(p->CBtermH));
+		readParam(rcfp, "CBPHONE", 'S', 0, MAXLEN_PHONE, p->CBphone);
 		readParam(rcfp, "MODEMDEVICE", 'S', 0, MAXLEN_DEVICE, p->modemDevice);
 		line = readParam(rcfp, "MODEMSPEED", 'S', 4, 6, buf);
 		if (! (n=str2speed(buf)))
@@ -422,7 +435,7 @@ static void inputAllXisprc(xisprc_t *p, FILE *rcfp, glob_t *global)
 		readParam(rcfp, "MODEMRESET", 'S', 0, MAXLEN_MDMCMD, p->modemReset);
 		readParam(rcfp, "MODEMINIT", 'S', 0, MAXLEN_MDMCMD, p->modemInit);
 		readParam(rcfp, "MODEMCONNECT", 'S',0, MAXLEN_MDMSTR, p->modemConnect);
-		readParam(rcfp, "OPEROPTS", 'X', 0, 0, &(p->operOpts));
+		readParam(rcfp, "OPEROPTS", 'L', 0, 0, &(p->operOpts));
 		readParam(rcfp, "DIALEXTRA", 'S', 0, MAXLEN_DIALEXTRA, p->dialExtra);
 		readParam(rcfp, "COMPLEVEL", 'B', 9, 15, &(p->compLevel));
 		readParam(rcfp, "ASYNCMAP", 'S', 2, MAXDIG_ASYNCMAP, p->asyncmap);
@@ -488,6 +501,9 @@ void initXisprc(xisprc_t *p)
 	p->CBDelay = MAXSEC_CBDELAY;
 	p->termW = TERMW;
 	p->termH = TERMH;
+	p->CBtermW = TERMW;
+	p->CBtermH = TERMH;
+	p->CBphone[0] = 0;
 	strcpy(p->modemDevice, MODEM_DEVICE);
 	p->modemSpeed = MODEM_SPEED;
 	strcpy(p->modemReset, MODEM_RESET);
