@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998  Dimitrios P. Bouras
+  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@ void doQuit(FL_OBJECT *, long);
 void doOptions(FL_OBJECT *, long);
 void doHelp(FL_OBJECT *, long);
 void doISPDropChoice(FL_OBJECT *, long);
+void doStartupTip(FL_OBJECT *, long);
 
 void doTelNoInput(FL_OBJECT *, long);
 void doAccountInput(FL_OBJECT *, long);
@@ -50,6 +51,7 @@ void doISPAutoDial(FL_OBJECT *, long);
 void doAutoRedial(FL_OBJECT *, long);
 void doAccountOK(FL_OBJECT *, long);
 void doAccountCancel(FL_OBJECT *, long);
+void doAccountTip(FL_OBJECT *, long);
 
 void doRetryInput(FL_OBJECT *, long);
 void doDelayInput(FL_OBJECT *, long);
@@ -57,6 +59,7 @@ void doCBYesButton(FL_OBJECT *, long);
 void doCBNoButton(FL_OBJECT *, long);
 void doCBOptions(FL_OBJECT *, long);
 void doCNWaitInput(FL_OBJECT *, long);
+void doCBInfoTip(FL_OBJECT *, long);
 void doRBYesButton(FL_OBJECT *, long);
 void doRBNoButton(FL_OBJECT *, long);
 void doTTYesButton(FL_OBJECT *, long);
@@ -73,6 +76,12 @@ void doCBInfoCancel(FL_OBJECT *, long);
 void doCBDelayInput(FL_OBJECT *, long);
 void doCBExpectInput(FL_OBJECT *, long);
 void doCBSendInput(FL_OBJECT *, long);
+void doCBColumnsInput(FL_OBJECT *, long);
+void doCBRowsInput(FL_OBJECT *, long);
+void doCBTelInput(FL_OBJECT *, long);
+void doLTScriptedButton(FL_OBJECT *, long);
+void doLTTerminalButton(FL_OBJECT *, long);
+void doLTRASButton(FL_OBJECT *, long);
 
 void doISPNameInput(FL_OBJECT *, long);
 void doISPNameEditOK(FL_OBJECT *, long);
@@ -93,9 +102,11 @@ void doModemResetInput(FL_OBJECT *, long);
 void doModemConnectInput(FL_OBJECT *, long);
 void doToneButton(FL_OBJECT *, long);
 void doPulseButton(FL_OBJECT *, long);
+void doISDNButton(FL_OBJECT *, long);
 void doModemDialInput(FL_OBJECT *, long);
-void doSWCOnButton(FL_OBJECT *, long);
+void doSWCBSDButton(FL_OBJECT *, long);
 void doSWCOffButton(FL_OBJECT *, long);
+void doSWCDeflateButton(FL_OBJECT *, long);
 void doSpeedButton(FL_OBJECT *, long);
 void doSWCInput(FL_OBJECT *, long);
 void doAsyncmapInput(FL_OBJECT *, long);
@@ -104,6 +115,7 @@ void doEscapeNoButton(FL_OBJECT *, long);
 void doEscapeInput(FL_OBJECT *, long);
 void doHWButton(FL_OBJECT *, long);
 void doSWButton(FL_OBJECT *, long);
+void doCommInfoTip(FL_OBJECT *, long);
 
 void doLocalIPInput(FL_OBJECT *, long);
 void doALYesButton(FL_OBJECT *, long);
@@ -112,7 +124,7 @@ void doRemoteIPInput(FL_OBJECT *, long);
 void doARYesButton(FL_OBJECT *, long);
 void doARNoButton(FL_OBJECT *, long);
 void doTcpipInfoOK(FL_OBJECT *, long);
-void doCommInfoCancel(FL_OBJECT *, long);
+void doTcpipInfoCancel(FL_OBJECT *, long);
 void doDRYesButton(FL_OBJECT *, long);
 void doDRNoButton(FL_OBJECT *, long);
 void doRemoteNMInput(FL_OBJECT *, long);
@@ -122,6 +134,7 @@ void doUDYesButton(FL_OBJECT *, long);
 void doUDNoButton(FL_OBJECT *, long);
 void doPDNSInput(FL_OBJECT *, long);
 void doSDNSInput(FL_OBJECT *, long);
+void doTcpipInfoTip(FL_OBJECT *, long);
 
 void doOLTimeButton(FL_OBJECT *, long);
 void doOLChargeButton(FL_OBJECT *, long);
@@ -198,6 +211,7 @@ void doRunDefault(FL_OBJECT *, long);
 void doChatDefault(FL_OBJECT *, long);
 void doXispUtilsDefault(FL_OBJECT *, long);
 void doPipeDefault(FL_OBJECT *, long);
+void doEnvInfoTip(FL_OBJECT *, long);
 
 /* Forms and Objects */
 
@@ -242,6 +256,7 @@ typedef struct {
 	FL_OBJECT *statusText;
 	FL_OBJECT *topIcon;
 	FL_OBJECT *ISPDropChoice;
+	FL_OBJECT *tipTimer;
 } FD_topFrame;
 FD_topFrame * create_form_topFrame(void);
 
@@ -270,6 +285,7 @@ typedef struct {
 	FL_OBJECT *ISPDefault;
 	FL_OBJECT *ISPAutoDial;
 	FL_OBJECT *autoRedial;
+	FL_OBJECT *tipTimer;
 } FD_accountInfo;
 FD_accountInfo * create_form_accountInfo(void);
 
@@ -310,6 +326,7 @@ typedef struct {
 	FL_OBJECT *sendInput;
 	FL_OBJECT *dialOK;
 	FL_OBJECT *dialCancel;
+	FL_OBJECT *tipTimer;
 } FD_dialInfo;
 FD_dialInfo * create_form_dialInfo(void);
 
@@ -317,12 +334,22 @@ typedef struct {
 	FL_FORM *CBInfo;
 	void *vdata;
 	char *cdata;
-	long ldata;
+	long  ldata;
 	FL_OBJECT *CBInfoOK;
 	FL_OBJECT *CBInfoCancel;
 	FL_OBJECT *CBdlyInput;
+	FL_OBJECT *LTGroup;
+	FL_OBJECT *LTScriptedButton;
+	FL_OBJECT *LTTerminalButton;
+	FL_OBJECT *LTRASButton;
+	FL_OBJECT *ALGroup;
 	FL_OBJECT *CBExpectInput;
 	FL_OBJECT *CBSendInput;
+	FL_OBJECT *TSGroup;
+	FL_OBJECT *columnsInput;
+	FL_OBJECT *rowsInput;
+	FL_OBJECT *CBTelInput;
+	FL_OBJECT *tipTimer;
 } FD_CBInfo;
 FD_CBInfo * create_form_CBInfo(void);
 
@@ -364,6 +391,7 @@ typedef struct {
 	long  ldata;
 	FL_OBJECT *alertMessageDismiss;
 	FL_OBJECT *alertMessageStr;
+	FL_OBJECT *noMoreHints;
 } FD_alertMessage;
 FD_alertMessage * create_form_alertMessage(void);
 
@@ -380,9 +408,11 @@ typedef struct {
 	FL_OBJECT *modemDialType;
 	FL_OBJECT *toneButton;
 	FL_OBJECT *pulseButton;
+	FL_OBJECT *ISDNButton;
 	FL_OBJECT *modemDialInput;
 	FL_OBJECT *SWCType;
-	FL_OBJECT *SWCOnButton;
+	FL_OBJECT *SWCBSDButton;
+	FL_OBJECT *SWCDeflateButton;
 	FL_OBJECT *SWCOffButton;
 	FL_OBJECT *deviceInput;
 	FL_OBJECT *modemSpeedType;
@@ -396,6 +426,7 @@ typedef struct {
 	FL_OBJECT *flowCtrlType;
 	FL_OBJECT *HWButton;
 	FL_OBJECT *SWButton;
+	FL_OBJECT *tipTimer;
 } FD_commInfo;
 FD_commInfo * create_form_commInfo(void);
 
@@ -425,6 +456,7 @@ typedef struct {
 	FL_OBJECT *MRUInput;
 	FL_OBJECT *pDNSInput;
 	FL_OBJECT *sDNSInput;
+	FL_OBJECT *tipTimer;
 } FD_tcpipInfo;
 FD_tcpipInfo * create_form_tcpipInfo(void);
 
@@ -459,6 +491,7 @@ typedef struct {
 	long ldata;
 	FL_OBJECT *statsBrowser;
 	FL_OBJECT *statsChart;
+	FL_OBJECT *pieChart;
 	FL_OBJECT *statInfoOK;
 } FD_statInfo;
 FD_statInfo * create_form_statInfo(void);
@@ -599,6 +632,7 @@ typedef struct {
 	FL_OBJECT *runStatus;
 	FL_OBJECT *chatStatus;
 	FL_OBJECT *utilsStatus;
+	FL_OBJECT *tipTimer;
 } FD_envInfo;
 FD_envInfo * create_form_envInfo(void);
 
@@ -612,6 +646,7 @@ FD_envInfo * create_form_envInfo(void);
 #ifdef XPMANIMATE
 #define XPM_INTERVAL  0.3	/* XPM animation trigger interval */
 #endif
+#define HINT_TRIGGER  0.2	/* one-shot trigger interval for pop-up hints */
 
 typedef struct {			/* the color structure */
 	int r, g, b;
