@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998  Dimitrios P. Bouras
+  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <varargs.h>
+#include <sys/param.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/types.h>
@@ -38,6 +39,7 @@ extern char *sys_errlist[];
 extern int errno;
 extern int fputs(), fscanf(), fprintf(), fclose();
 extern char *vsprintf();
+extern void bcopy();
 #endif
 
 /*+-------------------------------------------------------------------------+
@@ -59,6 +61,7 @@ int connectWait = MAXSEC_CNWAIT;		/* maximum wait for connection */
 char account[MAXLEN_ACCOUNT+2];			/* account name */
 char **phone;							/* phone numbers */
 int numPhones;							/* how many there are to dial */
+unsigned int mode;						/* connection mode */
 char **sline;							/* script lines */
 int numSlines;							/* how many there are to include */
 char **CBsln;							/* call-back script lines */
@@ -69,12 +72,8 @@ char modemUInit[MAXLEN_MDMCMD+2];		/* user specified modem init str */
 char modemUReset[MAXLEN_MDMCMD+2];		/* user specified modem reset string */				
 char modemUConn[MAXLEN_MDMSTR+2];		/* user specified modem connect str */
 char modemDial[MAXLEN_DIALEXTRA+2+2];	/* dialing chars + method char */
-int autologin = 1;						/* true for automatic login */
-int callback = 0;						/* true when call-back desired */
-int manual = 0;							/* true for manual terminal */
 char dispname[MAXLEN_TTPARM+2];			/* X11 display for auto login */
 char geometry[MAXLEN_TTPARM+2];			/* and terminal window geometry */
-int authlogin;							/* true for PAP/PAPS/CHAPS login */
 char bgcol[16] = {0};					/* form-background color string */
 
 
@@ -93,13 +92,13 @@ void doErr(char *msg)
 {
 	char emsg[MSGLEN_ERR];
 
-#ifndef SUNOS5x
+#ifdef HAVE_STRERROR
+	sprintf(emsg, "xispdial: %s: %s\n", msg, strerror(errno));
+#else
 	if (errno < sys_nerr)
 		sprintf(emsg, "xispdial: %s: %s\n", msg, sys_errlist[errno]);
 	else
 		sprintf(emsg, "xispdial: %s: error #%d\n", msg, errno);
-#else
-	sprintf(emsg, "xispdial: %s: %s\n", msg, strerror(errno));
 #endif
 	fputs(emsg, stderr);
 	if (pipeFD) close(pipeFD);
@@ -121,10 +120,12 @@ int namedPipe(char *fname)
 		if (!S_ISFIFO(st.st_mode))				/* is it a FIFO? */
 			doErr("namedPipe: stat");			/* nope, still not right! */
 	}
-#ifndef SUNOS5x
-	fd = open(fname, O_WRONLY|O_NDELAY);		/* yes, open it for writing */
-#else
+#ifdef SUNOS5x
 	fd = open(fname, O_WRONLY|O_NONBLOCK);		/* yes, open it for writing */
+#elif (defined(BSD) && BSD >= 199306)
+	fd = open(fname, O_WRONLY|O_NONBLOCK);
+#else
+	fd = open(fname, O_WRONLY|O_NDELAY);
 #endif
 	if (fd < 0)									/* error means no process has */
 		doErr("namedPipe: open");				/* opened it for reading */
@@ -145,11 +146,11 @@ int pprintf(va_alist) va_dcl
 
 	va_start(ap);								/* start variable arg list */
 	fmt = va_arg(ap, char*);					/* first string is format */
-#ifndef SUNOS41x
-	iw = vsprintf(msg, fmt, ap);				/* pass rest to vsprintf() */
-#else
-	vsprintf(msg, fmt, ap);
+#ifdef BROKEN_VSPRINTF
+	vsprintf(msg, fmt, ap);						/* pass rest to vsprintf() */
 	iw = strlen(msg);
+#else
+	iw = vsprintf(msg, fmt, ap);
 #endif
 	va_end(ap);									/* end variable arg list */
 	bw = write(pipeFD, msg, strlen(msg));		/* write buffer to pipe */
@@ -202,9 +203,10 @@ int child(va_alist) va_dcl
 
 void outOfMem(char *msg)
 {
-	fprintf(stderr, "xispdial: %s: out of memory!\n", msg);
-	if (pipeFD) close(pipeFD);
-	exit(1);
+	fprintf(stderr, "%s: %s: out of memory!\n",	/* notify xisp */
+			"xispdial", msg);
+	if (pipeFD) close(pipeFD);					/* close the report pipe */
+	exit(1);									/* and exit with error */
 }
 
 /* Initializes the dialer file names using the user home directory. */
@@ -226,7 +228,8 @@ void initFnames(void)
 }
 
 
-/* Reads the environment file created by xISP and then deletes it */
+/* getISPenv reads the environment file created by xISP, adjusts
+   xispdial's operation mode flags and and then deletes the file. */
 
 void uEOF(void)
 {
@@ -239,47 +242,45 @@ void uEOF(void)
 FILE *envfp;  /* environment file pointer common to readLine(), getISPenv() */
 
 /* The readLine function reads a single line from the environment file
-   prepared by xisp and before returning can be instructed to either escape
-   all '\\' characters, or to delete them from the buffer. It assumes that
-   the buffer has the space required for escaping all '\\' characters found. */
+   prepared by xisp, and can be instructed to remove all '\x' escape
+   sequences from the buffer before returning */
 
-void readLine(char *param, int len, int escape)
+void readLine(char *param, int len, int allowEscape)
 {
 	char *p;
-	int zlen;	/* line length (includes ending '\x0') */
+	int rest;
 
 	if (fgets(param, len+1,						/* read environment line */
 			  envfp) == NULL)
 		uEOF();									/* bail out on error */
-	param[(zlen = strlen(param))-1]=0;			/* kill the ending '\n' */
+	param[rest = strlen(param)-1] = 0;			/* kill the ending '\n' */
 	for (p=param; *p;) {						/* scan the line */
-		if (*p == '\\') {						/* if a '\\' is found */
-			if (escape) {						/* if auto escaping selected */
-#ifndef SUNOS41x
-				memmove(p+1, p, zlen);			/* increase string len by 1 */
-#else											/* by duplicating the string */
-				bcopy(p, p+1, zlen);			/* contents starting from */
-#endif											/* next character position */
-				*p = '\\';						/* insert an extra '\\' */
-				p += 2;							/* skip the next '\\' */
-			}
-			else {								/* deletion of '\\'s desired */
-#ifndef SUNOS41x
-				memmove(p, p+1, zlen);			/* decrease string len by 1 */
-#else
-				bcopy(p+1, p, zlen);
+		if (*p == '\\' && !allowEscape) {		/* if '\\' is found and and */
+#ifdef SUNOS41x									/* escape chars not allowed */
+			bcopy(p+1, p, rest);				/* eat-up the '\\' by */
+#else											/* decreasing string len by 1 */
+			memmove(p, p+1, rest);
 #endif
+			--rest;								/* adjust remaining chars */
+			if (*p) {							/* if it's not the last */
+#ifdef SUNOS41x									/* char in the string, then */
+				bcopy(p+1, p, rest);			/* eat-up the next char also */
+#else
+				memmove(p, p+1, rest);
+#endif
+				--rest;							/* adjust remaining chars */
 			}
 		}
-		else									/* char not a '\\' */
+		else {									/* char not a '\\' */
 			++p;								/* so carry on */
+			--rest;								/* adjust remaining chars */
+		}
 	}
 }
 
 void getISPenv(void)
 {
 	int ir, i;
-	char LM[16] = {0};
 
 	envfp = fopen(envfname, "r");				/* open temp environment file */
 	if (envfp == NULL)							/* if not there bail out */
@@ -310,48 +311,69 @@ void getISPenv(void)
 		outOfMem("getISPenv");					/* nope, bail out */
 	for (i=0; i<numPhones; i++)
 		readLine(phone[i], MAXLEN_PHONE, 0);	/* read telephone numbers */
-	readLine(LM, (sizeof LM)-2, 0);				/* read login method */
-	autologin = (strstr(LM, "AUTO") != NULL);	/* adjust autologin flag */
-	if (autologin)
-		callback = !strcmp(LM, "AUTO-CB");		/* adjust call-back flag */
-	else {
-		authlogin = (strstr(LM, "AUTH")!=NULL);	/* adjust authlogin flag */
-		if (authlogin) {
-			callback = !strcmp(LM, "AUTH-CB");	/* adjust call-back flag */
-			manual = !strcmp(LM, "AUTH-MAN");	/* adjust manual flag */
-		}
-		else
-			manual = 1;
-	}
+	if (fscanf(envfp,"%X ",&mode) < 1)			/* read connection method */
+		uEOF();
+	switch (mode) {
 
-	if (autologin) {							/* if login is indeed auto */
-		readLine(account, MAXLEN_ACCOUNT, 0);	/* read account name and */
-		ir = fscanf(envfp,"%d ",&numSlines);	/* number of script lines */
-		if (! ir)								/* bail out on error */
-			uEOF();
-		if (numSlines) {						/* 0 lines is possible */
-			sline = (char **)					/* allocate space for */
-			 malloc(numSlines*sizeof(char *));	/* numSlines scipt lines */
-			for (i=0; i<numSlines&&sline!=NULL;	/* which follow */
-				 i++)
-			{
-				sline[i] = (char *)				/* allocate script line */
-					malloc(4*MAXLEN_SLINE+10	/* double the size for */
-						  +5*MAXLEN_ACCOUNT+2);	/* automatic escaping */
-				if (sline[i] != NULL)
-					*(sline[i]) = 0;			/* and initialize it */
-				else
-					sline = NULL;
+		case SCRIPT_DIALIN:
+			readLine(account,MAXLEN_ACCOUNT,0);	/* read account name and */
+			ir= fscanf(envfp,"%d ",&numSlines);	/* number of script lines */
+			if (! ir)							/* bail out on error */
+				uEOF();
+			if (numSlines) {					/* 0 lines is possible */
+				sline = (char **)				/* allocate space for */
+				  malloc(numSlines*				/* numSlines scipt lines */
+						 sizeof(char *));
+				for (i=0;
+					 i<numSlines&&sline!=NULL;	/* which follow */
+					 i++)
+				{
+					sline[i] = (char *)			/* allocate script line */
+					  malloc(4*MAXLEN_SLINE+10	/* double the size for */
+						+5*MAXLEN_ACCOUNT+2);	/* automatic escaping */
+					if (sline[i] != NULL)
+						*(sline[i]) = 0;		/* and initialize it */
+					else
+						sline = NULL;
+				}
+				if (sline == NULL)				/* allocation completed? */
+					outOfMem("getISPenv");		/* nope, bail out */
+				for (i=0; i<numSlines; i++)
+					readLine(sline[i],			/* read script lines */
+							 4*MAXLEN_SLINE+10+
+							 5*MAXLEN_ACCOUNT, 1);
 			}
-			if (sline == NULL)					/* allocation completed? */
-				outOfMem("getISPenv");			/* nope, bail out */
-			for (i=0; i<numSlines; i++)
-				readLine(sline[i],				/* read script lines */
-						 4*MAXLEN_SLINE+10+
-						 5*MAXLEN_ACCOUNT, 1);
-		}
-		if (callback) {
-			ir= fscanf(envfp,"%d ",&CBDelay);	/* time wait for call-back */
+		break;
+
+		case SCRIPT_DIALIN|SCRIPT_CALLBACK:
+			readLine(account,MAXLEN_ACCOUNT,0);	/* read account name and */
+			ir= fscanf(envfp,"%d ",&numSlines);	/* number of script lines */
+			if (! ir)							/* bail out on error */
+				uEOF();
+			if (numSlines) {					/* 0 lines is possible */
+				sline = (char **)				/* allocate space for */
+				  malloc(numSlines*				/* numSlines scipt lines */
+						 sizeof(char *));
+				for (i=0;
+					 i<numSlines&&sline!=NULL;	/* which follow */
+					 i++)
+				{
+					sline[i] = (char *)			/* allocate script line */
+					  malloc(4*MAXLEN_SLINE+10	/* double the size for */
+						+5*MAXLEN_ACCOUNT+2);	/* automatic escaping */
+					if (sline[i] != NULL)
+						*(sline[i]) = 0;		/* and initialize it */
+					else
+						sline = NULL;
+				}
+				if (sline == NULL)				/* allocation completed? */
+					outOfMem("getISPenv");		/* nope, bail out */
+				for (i=0; i<numSlines; i++)
+					readLine(sline[i],			/* read script lines */
+							 4*MAXLEN_SLINE+10+
+							 5*MAXLEN_ACCOUNT, 1);
+			}
+			ir = fscanf(envfp,"%d ",&CBDelay);	/* time wait for call-back */
 			ir+=fscanf(envfp,"%d ",&numCBSlns);	/* number of call-back lines */
 			if (ir < 2)							/* bail out on error */
 				uEOF();
@@ -364,8 +386,8 @@ void getISPenv(void)
 					 i++)
 				{
 					CBsln[i] = (char *)			/* allocate call-back line */
-						malloc(4*MAXLEN_SLINE+
-						 10+5*MAXLEN_ACCOUNT+2);
+					  malloc(4*MAXLEN_SLINE+
+						10+5*MAXLEN_ACCOUNT+2);
 					if (CBsln[i] != NULL)
 						*(CBsln[i]) = 0;		/* and initialize it */
 					else
@@ -378,28 +400,137 @@ void getISPenv(void)
 							 4*MAXLEN_SLINE+10+
 							 5*MAXLEN_ACCOUNT, 1);
 			}
-		}
-	}
-	else if (authlogin) {						/* auth-login? */
-		if (callback) {							/* yes, call-back desired? */
-			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
-				uEOF();							/* bail out on error */
-		}
-		else if (manual) {						/* manual terminal desired */
+		break;
+
+		case SCRIPT_DIALIN|MANUAL_CALLBACK:
+			readLine(account,MAXLEN_ACCOUNT,0);	/* read account name and */
+			ir= fscanf(envfp,"%d ",&numSlines);	/* number of script lines */
+			if (! ir)							/* bail out on error */
+				uEOF();
+			if (numSlines) {					/* 0 lines is possible */
+				sline = (char **)				/* allocate space for */
+				  malloc(numSlines*				/* numSlines scipt lines */
+						 sizeof(char *));
+				for (i=0;
+					 i<numSlines&&sline!=NULL;	/* which follow */
+					 i++)
+				{
+					sline[i] = (char *)			/* allocate script line */
+					  malloc(4*MAXLEN_SLINE+10	/* double the size for */
+						+5*MAXLEN_ACCOUNT+2);	/* automatic escaping */
+					if (sline[i] != NULL)
+						*(sline[i]) = 0;		/* and initialize it */
+					else
+						sline = NULL;
+				}
+				if (sline == NULL)				/* allocation completed? */
+					outOfMem("getISPenv");		/* nope, bail out */
+				for (i=0; i<numSlines; i++)
+					readLine(sline[i],			/* read script lines */
+							 4*MAXLEN_SLINE+10+
+							 5*MAXLEN_ACCOUNT, 1);
+			}
 			readLine(dispname,					/* read the X11 display */
 					 MAXLEN_TTPARM, 0);
 			readLine(bgcol,						/* read background color */
 					 (sizeof bgcol)-2, 0);
 			readLine(geometry,					/* read window geometry */
 					 MAXLEN_TTPARM, 0);
-		}
-	}
-	else {										/* else manual login desired */
-		readLine(dispname, MAXLEN_TTPARM, 0);	/* read the X11 display */
-		readLine(bgcol, (sizeof bgcol)-2, 0);	/* read background color */
-		readLine(geometry, MAXLEN_TTPARM, 0);	/* read window geometry */
-	}
+			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
+				uEOF();							/* bail out on error */
+		break;
 
+		case MANUAL_DIALIN:
+			readLine(dispname,					/* read the X11 display */
+					 MAXLEN_TTPARM, 0);
+			readLine(bgcol,						/* read background color */
+					 (sizeof bgcol)-2, 0);
+			readLine(geometry,					/* read window geometry */
+					 MAXLEN_TTPARM, 0);
+		break;
+
+		case MANUAL_DIALIN|SCRIPT_CALLBACK:
+			readLine(dispname,					/* read the X11 display */
+					 MAXLEN_TTPARM, 0);
+			readLine(bgcol,						/* read background color */
+					 (sizeof bgcol)-2, 0);
+			readLine(geometry,					/* read window geometry */
+					 MAXLEN_TTPARM, 0);
+			ir = fscanf(envfp,"%d ",&CBDelay);	/* time wait for call-back */
+			ir+=fscanf(envfp,"%d ",&numCBSlns);	/* number of call-back lines */
+			if (ir < 2)							/* bail out on error */
+				uEOF();
+			if (numCBSlns) {					/* 0 lines is possible */
+				CBsln = (char **)				/* allocate space for */
+				 malloc(numSlines*				/* numCBSlns call-back lines */
+						sizeof(char *));
+				for (i=0; i<numCBSlns &&		/* which follow */
+						  CBsln!=NULL;
+					 i++)
+				{
+					CBsln[i] = (char *)			/* allocate call-back line */
+					  malloc(4*MAXLEN_SLINE+
+						10+5*MAXLEN_ACCOUNT+2);
+					if (CBsln[i] != NULL)
+						*(CBsln[i]) = 0;		/* and initialize it */
+					else
+						CBsln = NULL;
+				}
+				if (CBsln == NULL)				/* allocation completed? */
+					outOfMem("getISPenv");		/* nope, bail out */
+				for (i=0; i<numCBSlns; i++)
+					readLine(CBsln[i],			/* read script lines */
+							 4*MAXLEN_SLINE+10+
+							 5*MAXLEN_ACCOUNT, 1);
+			}
+		break;
+
+		case MANUAL_DIALIN|MANUAL_CALLBACK:
+			readLine(dispname,					/* read the X11 display */
+					 MAXLEN_TTPARM, 0);
+			readLine(bgcol,						/* read background color */
+					 (sizeof bgcol)-2, 0);
+			readLine(geometry,					/* read window geometry */
+					 MAXLEN_TTPARM, 0);
+			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
+				uEOF();							/* bail out on error */
+		break;
+
+		case AUTH_DIALIN:
+		case AUTH_DIALIN|NT_RAS_DIALIN:
+		break;
+
+		case AUTH_DIALIN|AUTH_CALLBACK:
+		case AUTH_DIALIN|NT_RAS_CALLBACK:
+			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
+				uEOF();							/* bail out on error */
+		break;
+
+		case AUTH_DIALIN|MANUAL_DIALIN:
+		case AUTH_DIALIN|MANUAL_DIALIN|
+			 NT_RAS_DIALIN:
+			readLine(dispname,					/* read the X11 display */
+					 MAXLEN_TTPARM, 0);
+			readLine(bgcol,						/* read background color */
+					 (sizeof bgcol)-2, 0);
+			readLine(geometry,					/* read window geometry */
+					 MAXLEN_TTPARM, 0);
+		break;
+
+		case AUTH_DIALIN|MANUAL_DIALIN|
+			 AUTH_CALLBACK:
+			readLine(dispname,					/* read the X11 display */
+					 MAXLEN_TTPARM, 0);
+			readLine(bgcol,						/* read background color */
+					 (sizeof bgcol)-2, 0);
+			readLine(geometry,					/* read window geometry */
+					 MAXLEN_TTPARM, 0);
+			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
+				uEOF();							/* bail out on error */
+		break;
+
+		default: break;
+	}
 	readLine(modemUReset, MAXLEN_MDMCMD, 1);	/* modem reset string */
 	readLine(modemUInit, MAXLEN_MDMCMD, 1);		/* modem init string */
 	readLine(modemDial, MAXLEN_DIALEXTRA+2, 0);	/* dial char(s) + method */
@@ -443,89 +574,236 @@ int init_modem(char *initCmd)					/* initialize modem */
 	return rcode;
 }
 
+/* Dial the number given and process the connection mode selected */
+
 #define MAXLEN_PREAMBLE 512
 #define MAXLEN_CBTRAIL  256
+#define MAXLEN_SCRIPT   (MAXLEN_PREAMBLE + MAXLEN_DIALEXTRA +MAXLEN_PHONE + \
+						 MAXDIG_CNWAIT + MAXLEN_MDMSTR + \
+                         2*MAXNUM_SLINES * (MAXLEN_SLINE + MAXLEN_ACCOUNT) + \
+                         MAXLEN_CBTRAIL + MAXDIG_CNWAIT + MAXLEN_MDMSTR)
 
-int callnumber(char *number)					/* dial the number given */
+int callnumber(char *number)
 {
 	int i, rcode;
 	FILE *scriptF;
-	char preambleD[MAXLEN_PREAMBLE+1] =
+	char preambleD[MAXLEN_PREAMBLE + 1] =
 			"TIMEOUT\t\t3\nABORT\t\tBUSY\nABORT\t\t'NO CARRIER'\n"
 			"ABORT\t\t'NO DIALTONE'\nABORT\t\tenied\nABORT\t\timeout\n"
 			"''\t\t\t'AT %s %s'\nTIMEOUT\t\t%d\n%s\t\t\\c\n"
 			"TIMEOUT\t\t5\n\\r\t\t\\c\n",
-		 preambleCB[MAXLEN_PREAMBLE+1] =
+		 preambleCB[MAXLEN_PREAMBLE + 1] =
 			"TIMEOUT\t\t3\nABORT\t\tBUSY\nABORT 'NO DIALTONE'\n"
 			"ABORT\t\tenied\nABORT\t\timeout\n''\t\t\t'AT %s %s'\n"
 			"TIMEOUT\t\t%d\n%s\t\t\\c\nTIMEOUT\t\t5\n\\r\t\t\\c\n",
 		 cbtrail[MAXLEN_CBTRAIL+1] =
-			"TIMEOUT\t\t%d\n%s\t\t\\c\nTIMEOUT\t\t5\n\\r\t\t\\c\n",
-		 script[MAXNUM_SLINES*(2*MAXLEN_SLINE+5)+MAXLEN_PREAMBLE+1];
+			"TIMEOUT\t\t%d\nRING\t\tATA\nTIMEOUT\t\t30\n%s\t\t\\c\n"
+			"TIMEOUT\t\t5\n\\r\t\t\\c\n",
+		 script[MAXLEN_SCRIPT + 1];
 
 	umask(077);									/* read/write only by owner */
 	scriptF = fopen(scriptfname, "w");			/* open the script file */
 	if (scriptF == NULL)						/* failed to do so? */
 		doErr("callnumber: fopen");				/* yup, bail out */
-	if (autologin) {							/* if automatic login */
-		if (callback)							/* and if call-back enabled */
+	switch (mode) {
+		case SCRIPT_DIALIN:
+			strcpy(script, preambleD);			/* normal dialing preamble */
+			for (i=0; i<numSlines; i++) {		/* append the script lines */
+				strcat(script, sline[i]);
+				strcat(script, "\n");
+			}
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
+					modemUConn);
+			break;
+
+		case SCRIPT_DIALIN|SCRIPT_CALLBACK:
 			strcpy(script, preambleCB);			/* copy call-back preamble */
-		else									/* otherwise copy */
-			strcpy(script, preambleD);			/* the dialing preamble */
-		for (i=0; i<numSlines; i++) {			/* append the script lines */
-			strcat(script, sline[i]);
-			strcat(script, "\n");
-		}
-		fprintf(scriptF, script, modemDial,		/* using the script as fmt */
-				number, connectWait,			/* stick all the rest in */
-				modemUConn);
-		if (callback) {							/* if call-back is desired */
-			strcpy(script, cbtrail);			/* to the script trail add */
-			for (i=0; i<numCBSlns; i++) {		/* the call-back lines */
+			for (i=0; i<numSlines; i++) {		/* append the script lines */
+				strcat(script, sline[i]);
+				strcat(script, "\n");
+			}
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
+					modemUConn);
+			strcpy(script, cbtrail);			/* add the call-back trail */
+			for (i=0; i<numCBSlns; i++) {		/* append call-back lines */
 				strcat(script, CBsln[i]);
 				strcat(script, "\n");
 			}
 			fprintf(scriptF, script,			/* using script as format */
-					CBDelay, modemUConn);		/* stick the rest in */
-		}
-	}
-	else {										/* PAP/PAPS/CHAPS or manual */
-		if (authlogin && callback) {			/* auth-login and call-back? */
-			strcpy(script, preambleCB);			/* copy the dialing preamble */
-			fprintf(scriptF, script, modemDial,	/* print dial-in script out */
-					number, connectWait,
+					CBDelay, modemUConn);		/* insert all parameters */
+			break;
+
+		case SCRIPT_DIALIN|MANUAL_CALLBACK:
+			strcpy(script, preambleCB);			/* copy call-back preamble */
+			for (i=0; i<numSlines; i++) {		/* append the script lines */
+				strcat(script, sline[i]);
+				strcat(script, "\n");
+			}
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
 					modemUConn);
-			strcpy(script, cbtrail);			/* copy the call-back trail */
-			fprintf(scriptF, script, CBDelay,	/* and print the rest out */
+			break;
+
+		case MANUAL_DIALIN:
+			strcpy(script, preambleD);			/* normal dialing preamble */
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
 					modemUConn);
-		}
-		else {									/* auth, auth-manual, manual */
-			strcpy(script, preambleD);			/* copy the dialing preamble */
-			fprintf(scriptF, script, modemDial,	/* and print the script out */
-					number, connectWait,
+			break;
+
+		case MANUAL_DIALIN|SCRIPT_CALLBACK:
+			strcpy(script, preambleCB);			/* copy call-back preamble */
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
 					modemUConn);
-		}
+			break;
+
+		case MANUAL_DIALIN|MANUAL_CALLBACK:
+			strcpy(script, preambleCB);			/* copy call-back preamble */
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
+					modemUConn);
+			break;
+
+		case AUTH_DIALIN:
+		case AUTH_DIALIN|NT_RAS_DIALIN:
+			strcpy(script, preambleD);			/* normal dialing preamble */
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
+					modemUConn);
+			break;
+
+		case AUTH_DIALIN|AUTH_CALLBACK:
+			strcpy(script, preambleCB);			/* copy call-back preamble */
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
+					modemUConn);
+			strcpy(script, cbtrail);			/* add the call-back trail */
+			fprintf(scriptF, script,			/* using script as format */
+					CBDelay, modemUConn);		/* insert all parameters */
+			break;
+
+		case AUTH_DIALIN|NT_RAS_CALLBACK:
+			strcpy(script, cbtrail);			/* copy call-back trail */
+			fprintf(scriptF, script,			/* using script as format */
+					CBDelay, modemUConn);		/* insert all parameters */
+			break;
+
+		case AUTH_DIALIN|MANUAL_DIALIN:
+		case AUTH_DIALIN|MANUAL_DIALIN|
+			 NT_RAS_DIALIN:
+			strcpy(script, preambleD);			/* normal dialing preamble */
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
+					modemUConn);
+			break;
+
+		case AUTH_DIALIN|MANUAL_DIALIN|
+			 AUTH_CALLBACK:
+			strcpy(script, preambleCB);			/* copy call-back preamble */
+			fprintf(scriptF, script, modemDial,	/* using script as format */
+					number, connectWait,		/* insert all parameters */
+					modemUConn);
+			break;
+
+		default:								/* no valid mode selected! */
+			pprintf("\n%s: %s\n", "xispdial",	/* so report it */
+					"invalid connection mode!");
+			close(pipeFD);						/* close the report pipe */
+			exit(1);							/* and bail out */
+			break;
 	}
 	fclose(scriptF);							/* close the script file */
 	rcode = child(Chat,"-V","-f",				/* hand it over to chat */
 				  scriptfname,(char *)0);
 	unlink(scriptfname);						/* done with script, delete */
 	if (rcode == 0) {							/* chat terminated OK? */
-		pprintf("\n%s: %s %s\n", "xispdial",	/* yes, send "done" message */
-				"done dialing", number);
-		if (manual && !callback)				/* start manual login window */
-			rcode = child(Terminal, "-bgcol",	/* if it is so desired */
-						bgcol, "-display",
-					  	dispname, "-geometry",
-						geometry, (char *)0);
+		if (mode & NT_RAS_CALLBACK)				/* send "done" message */
+			pprintf("\n%s: %s\n","xispdial",	/* according to type */
+					"done RAS call-back");
+		else
+			pprintf("\n%s: %s %s\n","xispdial",
+					"done dialing", number);
+		if (mode & (MANUAL_DIALIN |				/* carry on if manual login */
+					MANUAL_CALLBACK)) {			/* selected for either phase */
+			if (mode & MANUAL_DIALIN) {			/* if manual login selected */
+				if (child(Terminal, "-bgcol",	/* for dial-in phase, then */
+						  bgcol, "-display",	/* start terminal app  */
+						  dispname, "-geometry",
+						  geometry, (char *)0))	/* if terminal aborted */
+				{
+					close(pipeFD);				/* then close report pipe */
+					exit(1);					/* tell pppd: connect failed */
+				}
+			}
+			if (mode & (SCRIPT_CALLBACK |		/* if some type of */
+						MANUAL_CALLBACK |		/* call-back is selected */
+						AUTH_CALLBACK)) {
+				scriptF=fopen(scriptfname,"w");	/* create a new script file */
+				if (scriptF == NULL)			/* failed to do so? */
+					doErr("callnumber: fopen");	/* yup, bail out */
+				strcpy(script, cbtrail);		/* use call-back trail only */
+				if (mode & SCRIPT_CALLBACK) {	/* if call-back is scripted */
+					for (i=0;i<numCBSlns;i++) {	/* append call-back lines */
+					  strcat(script, CBsln[i]);
+					  strcat(script, "\n");
+					}
+				}
+				fprintf(scriptF, script, 		/* using script as format */
+						CBDelay, modemUConn);	/* insert all parameters */
+				fclose(scriptF);				/* close the script file */
+				child(Chat,"TIMEOUT","1",		/* swallow whatever's left */
+					  "NO CARRIER","\\c",		/* from the terminal session */
+					  (char *)0);
+				init_modem(modemInit);			/* re-initialize the modem */
+				rcode = child(Chat,"-V","-f",	/* hand script over to chat */
+							  scriptfname,
+							  (char *)0);
+				unlink(scriptfname);			/* done with script, delete */
+			}
+			if (rcode == 0 &&					/* if we're still OK and */
+				(mode & MANUAL_CALLBACK)) {		/* manual login selected */
+				if (child(Terminal, "-bgcol",	/* for call-back phase */
+						  bgcol, "-display",	/* then start terminal app */
+						  dispname, "-geometry",
+						  geometry, (char *)0))	/* if terminal aborted */
+				{
+					close(pipeFD);				/* then close report pipe */
+					exit(1);					/* tell pppd: connect failed */
+				}
+			}
+			if (rcode != 0 ) {					/* check what happened */
+				if (rcode < 0) {				/* problem with chat? */
+					pprintf("\n%s: %s\n",		/* yes, report it */
+					 "xispdial", "chat"
+					 "execution failed!");
+					close(pipeFD);				/* close the report pipe */
+					exit(1);					/* and bail out */
+				}
+				return rcode;					/* else valid report code */
+			}
+		}
+
+/*+-------------------------------------------------------------------------+
+  |                                                                         |
+  |  This is the only point in xispdial where we can exit with status 0,    |
+  |  i.e the only case in which status returned to pppd indicates success.  |
+  |                                                                         |
+  +-------------------------------------------------------------------------+*/
+
+		if (mode & NT_RAS_DIALIN)				/* NT-RAS call-back -> send */
+			pprintf("\nxispdial: NT-RAS\n");	/* trigger for 2nd "Connect" */
 		close(pipeFD);							/* close the report pipe */
 		exit(rcode);							/* and exit */
+
 	}
-	if (rcode < 0) {							/* nope, chat problem */
-		pprintf("\n%s: %s\n", "xispdial",		/* report it */
+	if (rcode < 0) {							/* problem with chat? */
+		pprintf("\n%s: %s\n", "xispdial",		/* yes, report it */
 				"chat execution failed!");
 		close(pipeFD);							/* and bail out */
-		exit(1);							
+		exit(1);
 	}
 	return rcode;								/* else valid report code */
 }
@@ -589,8 +867,10 @@ int main()
 	pipeFD = namedPipe(Pipe);		/* initialize named pipe to xISP */
 
 	pprintf("xispdial: PID=%d.\n", getpid());	/* the '.' is important! */
-	if (authlogin)
+	if (mode & AUTH_DIALIN)
 		pprintf("xispdial: dialing ISP: using PAP/CHAP\n");
+	else if (mode & MANUAL_DIALIN)
+		pprintf("xispdial: dialing ISP: using xispterm\n");
 	else
 		pprintf("xispdial: dialing ISP: user: %s\n", account);
 
@@ -607,6 +887,7 @@ int main()
 	if ((rcode=init_modem(modemInit))) {
 		pprintf("xispdial: chat returned %d!\n", rcode);
 		pprintf("xispdial: modem init failed!\n");
+		close(pipeFD);
 		return 1;
 	}
 
@@ -618,6 +899,7 @@ int main()
 		if ((rcode=init_modem(modemInit))) {
 			pprintf("xispdial: chat returned %d!\n", rcode);
 			pprintf("xispdial: modem re-init failed!\n");
+			close(pipeFD);
 			return 1;
 		}
 	}
