@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998  Dimitrios P. Bouras
+  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -88,13 +88,13 @@ void doErr(char *msg)
 {
 	char emsg[MSGLEN_ERR+1];
 
-#ifndef SUNOS5x
+#ifdef HAVE_STRERROR
+	sprintf(emsg, "xisprccv: %s: %s\n", msg, strerror(errno));
+#else
 	if (errno < sys_nerr)
 		sprintf(emsg, "xispterm: %s: %s\n", msg, sys_errlist[errno]);
 	else
 		sprintf(emsg, "xispterm: %s: error #%d\n", msg, errno);
-#else
-	sprintf(emsg, "xisprccv: %s: %s\n", msg, strerror(errno));
 #endif
 	fputs(emsg, stderr);
 	exit(1);
@@ -150,11 +150,11 @@ int bprintf(va_alist) va_dcl
 
 	va_start(ap);								/* start variable arg list */
 	fmt = va_arg(ap, char*);					/* first string is format */
-#ifndef SUNOS41x
-	bw = vsprintf(btmp, fmt, ap);				/* pass the rest to vsprintf */
-#else
-	vsprintf(btmp, fmt, ap);
+#ifdef BROKEN_VSPRINTF
+	vsprintf(btmp, fmt, ap);					/* pass the rest to vsprintf */
 	bw = strlen(btmp);
+#else
+	bw = vsprintf(btmp, fmt, ap);
 #endif
 	va_end(ap);									/* end variable arg list */
 	if ((tot+bw) < (MSGLEN_BROWSER-1))			/* do we have space for new? */
